<?php
/**
 * @version		$Id: view.html.php 20 2009-11-14 03:25:55Z dextercowley $
 * @package		com_fjrelated_plus
 * @copyright	Copyright (C) 2008 Mark Dexter. Portions Copyright Open Source Matters. All rights reserved.
 * @license		http://www.gnu.org/licenses/gpl.html
 *
 */

// no direct access

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.view');

/**
 * HTML View class for the FJ_Related Component
 *
 * @package    FJ_Related
 */

class FJRelatedViewFJRelated extends JView
{

	function __construct($config = array())
	{
		parent::__construct($config);

		//Add the helper path to the JHTML library
		JHTML::addIncludePath(JPATH_COMPONENT.DS.'helpers');
	}

	function display($tpl = null)
	{
		global $mainframe;
			
		$user		=& JFactory::getUser();
		$uri 		=& JFactory::getURI();
		$document	=& JFactory::getDocument();
		$dispatcher	=& JDispatcher::getInstance();
		$pathway	=& $mainframe->getPathway();

		// Get the menu item object
		$menus = &JSite::getMenu();
		$menu  = $menus->getActive();

		$params		=& $mainframe->getParams('com_content');

		// Initialize variables
		$article	=& $this->get('Article');
		$aparams	=& $article->parameters;
		$params->merge($aparams);

		// Request variables
		$layout     = JRequest::getCmd('layout');
		$task		= JRequest::getCmd('task');

		// Parameters
		$params->def('num_leading_articles', 	1);
		$params->def('num_intro_articles', 		4);
		$params->def('num_columns',				2);
		$params->def('num_links', 				4);
		$params->def('show_headings', 			1);
		$params->def('show_pagination',			2);
		$params->def('show_pagination_results',	1);
		$params->def('show_pagination_limit',	1);
		$params->def('filter',					1);

		$intro		= $params->get('num_intro_articles');
		$leading	= $params->get('num_leading_articles');
		$links		= $params->get('num_links');

		$limitstart	= JRequest::getVar('limitstart', 0, '', 'int');

		$params->def('display_num', $mainframe->getCfg('list_limit'));
		$default_limit = $params->get('display_num');

		if ($layout == 'blog') {
			$default_limit = $intro + $leading + $links;
		} else {
			$params->def('display_num', $mainframe->getCfg('list_limit'));
			$default_limit = $params->get('display_num');
		}

		$limit = $mainframe->getUserStateFromRequest('com_fjrelated.'.$this->getLayout().'.limit', 'limit', $default_limit, 'int');

		JRequest::setVar('limit', (int) $limit);

		$contentConfig = &JComponentHelper::getParams('com_content');
		$params->def('show_page_title', $contentConfig->get('show_title'));
		$menu_params = new JParameter( $menu->params );
		if (!$menu_params->get( 'page_title'))
		{
			$params->set('page_title',	($article->title) ? $article->title : $menu->name );
		}
		$document->setTitle( $params->get( 'page_title' ) );

		if ($article->metadesc) {
			$document->setDescription( $article->metadesc );
		}
		if ($article->metakey) {
			$document->setMetadata('keywords', $article->metakey);
		}

		$params->def('date_format',	JText::_('DATE_FORMAT_LC1'));

		// Get some data from the model
		$items		= & $this->get( 'Data' );
		if ($items) {
			$total = & $this->get( 'Total' ); // only do count if there are items to count
		} else {
			$total = 0;
		}
		//add alternate feed link
		if($params->get('show_feed_link', 1) == 1)
		{
			$link	= '&format=feed&limitstart=';
			$attribs = array('type' => 'application/rss+xml', 'title' => 'RSS 2.0');
			$document->addHeadLink(JRoute::_($link.'&type=rss'), 'alternate', 'rel', $attribs);
			$attribs = array('type' => 'application/atom+xml', 'title' => 'Atom 1.0');
			$document->addHeadLink(JRoute::_($link.'&type=atom'), 'alternate', 'rel', $attribs);
		}

		// Create a user access object for the user
		$access					= new stdClass();
		$access->canEdit		= $user->authorize('com_content', 'edit', 'content', 'all');
		$access->canEditOwn		= $user->authorize('com_content', 'edit', 'content', 'own');
		$access->canPublish		= $user->authorize('com_content', 'publish', 'content', 'all');

		jimport('joomla.html.pagination');
		//In case we are in a blog view set the limit
		if ($layout == 'blog') {
			$pagination = new JPagination($total, $limitstart, $limit - $links);
		} else {
			$pagination = new JPagination($total, $limitstart, $limit);
		}

		$this->assignRef('article', $article);
		$this->assignRef('params' , $params);
		$this->assignRef('user'   , $user);
		$this->assignRef('access',		$access);
		$this->assignRef('items'  , $items);
		$this->assign('total'  , $total);
		$this->assign('action', 	$uri->toString());
		$this->assignRef('pagination',	$pagination);

		parent::display($tpl);
	}

	/**
	 * Figure out what this does.
	 * @return unknown_type
	 */
	function &getItems()
	{
		global $mainframe;
		$params		=& $mainframe->getParams('com_content');
		$linkToFJRelated = $params->get('link_to_fjrelated', 0);
		//create select lists
		$user	= &JFactory::getUser();
		$lists	= $this->_buildSortLists();

		if (!count( $this->items ) )
		{
			$this->assign('lists',	$lists);
			$return = array();
			return $return;
		}

		//create paginatiion
		if ($lists['filter']) {
			$this->data->link .= '&amp;filter='.$lists['filter'];
		}

		$k = 0;
		$i = 0;
		$model = $this->getModel($this->_defaultModel);

		foreach($this->items as $key => $item)
		{
			// checks if the item is a public or registered/special item
			if ($item->access <= $user->get('aid', 0)) { // security ok
				// link to layout if param set and if there is a link found
				if (($linkToFJRelated) && ($relatedLink = $model->getRelatedLayoutLink($item->id, 'default'))) {
					$item->link = JRoute::_($relatedLink); // link to layout
					$item->link_type = 'fjrelated';
				}
				else { // otherwise link to article
					$item->link	= JRoute::_(ContentHelperRoute::getArticleRoute($item->slug, $item->catslug, $item->sectionid));
				}
				$item->readmore_register = false;
			}
			else  {
				$item->link = JRoute::_('index.php?option=com_user&task=register');
				$returnURL = JRoute::_(ContentHelperRoute::getArticleRoute($item->slug, $item->catslug, $item->sectionid));
				$fullURL = new JURI($item->link);
				$fullURL->setVar('return', base64_encode($returnURL));
				$item->link = $fullURL->toString();
				$item->readmore_register = true;
			}

			$item->created	= JHTML::_('date', $item->created, $this->params->get('date_format'));

			$item->odd		= $k;
			$item->count    = $i;

			$this->items[$key] = $item;
			$k = 1 - $k;
			$i++;
		}

		$this->assign('lists',	$lists);

		return $this->items;
	}

	function &getItem($index = 0, &$params)
	{
		global $mainframe;
		$params		=& $mainframe->getParams('com_content');
		$linkToFJRelated = $params->get('link_to_fjrelated', 0);
		$model = $this->getModel($this->_defaultModel);

		// Initialize some variables
		$user		=& JFactory::getUser();
		$dispatcher	=& JDispatcher::getInstance();

		$SiteName	= $mainframe->getCfg('sitename');

		$item 		=& $this->items[$index];
		$item->text = $item->introtext;

		// Get the page/component configuration and article parameters
		$item->params = clone($params);
		$aparams = new JParameter($item->attribs);

		// Merge article parameters into the page configuration
		$item->params->merge($aparams);

		// Process the content preparation plugins
		JPluginHelper::importPlugin('content');
		$results = $dispatcher->trigger('onPrepareContent', array (& $item, & $item->params, 0));

		// Build the link and text of the readmore button
		if (($item->params->get('show_readmore') && @ $item->readmore) || $item->params->get('link_titles'))
		{
			// checks if the item is a public or registered/special item
			if ($item->access <= $user->get('aid', 0)) {
				// link to FJ Related blog if param set and there is a link found
				if (($linkToFJRelated) && ($relatedLink = $model->getRelatedLayoutLink($item->id, 'blog'))) {
					$item->readmore_link = JRoute::_($relatedLink); // link to layout
					$item->link_type = 'fjrelated'; // set link type for CSS class
				}
				else {
					$item->readmore_link = JRoute::_(ContentHelperRoute::getArticleRoute($item->slug, $item->catslug, $item->sectionid));
				}
				$item->readmore_register = false;
			}
			else
			{
				$item->readmore_link = JRoute::_('index.php?option=com_user&view=login');
				$returnURL = JRoute::_(ContentHelperRoute::getArticleRoute($item->slug, $item->catslug, $item->sectionid));
				$fullURL = new JURI($item->readmore_link);
				$fullURL->setVar('return', base64_encode($returnURL));
				$item->readmore_link = $fullURL->toString();
				$item->readmore_register = true;
			}
		}

		$item->event = new stdClass();
		$results = $dispatcher->trigger('onAfterDisplayTitle', array (& $item, & $item->params,0));
		$item->event->afterDisplayTitle = trim(implode("\n", $results));

		$results = $dispatcher->trigger('onBeforeDisplayContent', array (& $item, & $item->params, 0));
		$item->event->beforeDisplayContent = trim(implode("\n", $results));

		$results = $dispatcher->trigger('onAfterDisplayContent', array (& $item, & $item->params, 0));
		$item->event->afterDisplayContent = trim(implode("\n", $results));

		return $item;
	}

	/**
	 * Figure out what this does
	 * @return unknown_type
	 */
	function _buildSortLists()
	{
		// Table ordering values
		$filter				= JRequest::getString('filter');
		global $mainframe;
		$itemid 			= JRequest::getInt('Itemid',0);
		$filter_order  		= $mainframe->getUserStateFromRequest('com_fjrelated.list.' . $itemid . ':.filter_order', 'filter_order', '', 'cmd');
		$filter_order_Dir 	= $mainframe->getUserStateFromRequest('com_fjrelated.list.' . $itemid . ':.filter_order_Dir', 'filter_order_Dir', '', 'cmd');

		$lists['task']      = 'fjrelated';
		$lists['filter']    = $filter;
		$lists['order']     = $filter_order;
		$lists['order_Dir'] = $filter_order_Dir;

		return $lists;
	}

}
?>